/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * Entscheidungstheorie - Prof.Dr.Peters - Master of Science
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 6.0
 * Semestergroup: IG2
 */

/**
 * USABILITY:
 *  Copyright (c) 2006
 *  Thomas Woellert (Dipl.-Inf. (FH)) (thomas@woellert.net)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package net.woellert.viergewinnt.player;

import java.util.ArrayList;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import net.woellert.viergewinnt.constants.ApplicationConstants;
import net.woellert.viergewinnt.objects.Playfield;
import net.woellert.viergewinnt.objects.Playfield.SlotState;
import net.woellert.viergewinnt.util.logging.LoggingManager;

/**
 * Class Description:
 * 		CPU player implementation via MiniMax-Algorithm
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 */
public class CPUPlayer extends AbstractPlayer {

	/* Our local logger object */
	protected static Logger localLogger = LoggingManager.getLogger(CPUPlayer.class.getName());

	// The max search depth of the tree
	protected int maxTreeDepth = 2;

	// The best turn found in the last procession run
	protected int bestTurnCol = -1;
	
	// The player representations for the MiniMax-Algorithm
	protected static final int NUM_PLAYER_1 = 1;
	protected static final int NUM_PLAYER_2 = -1;
	
	// Specific heuristic values
	protected final int HVALUE_TWO_MISSING = 10;
	protected final int HVALUE_ONE_MISSING = 100;
	protected final int HVALUE_FULL_GROUP = 10000;

	/**
	 * Constructor
	 * Use pre-defined play strength
	 */
	public CPUPlayer() {
		super();
		maxTreeDepth = 2;
	}

	/**
	 * Constructor
	 * @param level the level of the CPU player (1+)
	 */
	public CPUPlayer(int level) {
		super();
		maxTreeDepth = level;
	}

	@Override
	public void doTurn(Playfield field, SlotState playerState) {

		// Copy the playfield
		Playfield copyField = new Playfield(field);

		// Start the MiniMax algorithm
		long startTime = System.currentTimeMillis();

		int value = minMaxInit(copyField, playerState);

		long endTime = System.currentTimeMillis() - startTime;

		if (localLogger.isEnabledFor(Level.INFO)) {
			localLogger.log(Level.INFO, "CPUPlayer - MiniMax (depth=" + maxTreeDepth + ") complete: " + endTime + " ms");
			localLogger.log(Level.INFO, "Turn / Heuristic : " + bestTurnCol + " / " + value);
		}

		// Take the turn
		field.doTurn(bestTurnCol, playerState);

		// Reset the turn
		bestTurnCol = -1;
		
		try {
			Thread.currentThread().sleep(1000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Initialize the min-max-algorithm
	 * @param field the playfield
	 * @param playerState our player state
	 * @return the final heuristic
	 */
	protected int minMaxInit(Playfield field, SlotState playerState) {

		// To collect the best turns and heuristics
		ArrayList<Integer> bestTurns = new ArrayList<Integer>();
		ArrayList<Integer> bestTurnsHeuristic = new ArrayList<Integer>();

		// The determined best heuristic
		int bestHeuristic = 0;

		// Initialize the best heuristics
		if (playerState == SlotState.PLAYER_1) {

			// Player 1 = Maximizer, so start low
			bestHeuristic = Integer.MIN_VALUE;
		} else {

			// Player 2 = Minimizer, so start high
			bestHeuristic = Integer.MAX_VALUE;
		}

		// Determine all legal turns
		int[] allLegalTurnCols = field.getAllLegalTurns();

		// Go through all legal turns
		for (int colIndex = 0; colIndex < allLegalTurnCols.length; colIndex++) {
			
			// Determine the current turn
			int col = allLegalTurnCols[colIndex];
			int row = field.getMostBottomRow(col);

			// Set the turn
			field.setState(row, col, playerState);

			// Start the correct search depending on the player we are
			int heuristic = 0;

			if (playerState == SlotState.PLAYER_1) {
				
				// Player 1 is the maximizer
				heuristic = max(field, 0);
			} else {
				
				// Player 2 is the minimizer
				heuristic = min(field, 0);
			}

			// We got a heuristic, evaluate it depending on the player we are
			if (playerState == SlotState.PLAYER_1) {
				// We are the maximizer
				
				// Check if the best heuristic is smaller or equal than the new heuristic 
				if (bestHeuristic <= heuristic) {
					
					// We found a new max heuristic, remember it and its column
					bestHeuristic = heuristic;
					bestTurnCol = col;
				}
				
			} else {
				// We are the minimizer
				
				// Check if the best heuristic is greater or equal than the new heuristic
				if (bestHeuristic >= heuristic) {
					
					// We found a new min heuristic, remember it and its column
					bestHeuristic = heuristic;
					bestTurnCol = col;
				}
			}
			
			// Also remember both infos for the history
			bestTurnsHeuristic.add(heuristic);
			bestTurns.add(col);

			// Undo the turn
			field.setState(row, col, SlotState.EMPTY);
		}
		
		if (localLogger.isEnabledFor(Level.INFO)) {
			localLogger.log(Level.INFO, "CPUPlayer - First evaluation stage complete");
			localLogger.log(Level.INFO, "Determined best turns (col / heuristic) : ");
		}
		
		// Display all currently determined turns
		for (int i = 0; i < bestTurns.size(); i++) {
			
			// Mark the turn of choice with an arrow
			if(bestTurns.get(i) == bestTurnCol) {
				if (localLogger.isEnabledFor(Level.INFO)) {
					localLogger.log(Level.INFO, "     " + bestTurns.get(i) + " / " + bestTurnsHeuristic.get(i) + " <=");
				}
			} else {
				if (localLogger.isEnabledFor(Level.INFO)) {
					localLogger.log(Level.INFO, "     " + bestTurns.get(i) + " / " + bestTurnsHeuristic.get(i));
				}
			}
		}	
		
		// To collect the re-evaluated best turns and heuristics
		ArrayList<Integer> reBestTurns = new ArrayList<Integer>();
		ArrayList<Integer> reBestTurnsHeuristic = new ArrayList<Integer>();
		
		// Add another evaluation state to help deciding between turns with the same heuristic
		if(maxTreeDepth > 1) {
			// Only need to do this if we calculated more than one step in advance, else the results will be the same
			
			// Go through all remembered turns
			int newBestHeuristic = 0;
			int newBestTurn = -1;
			
			// Init values depending on our player number
			if(playerState == SlotState.PLAYER_1) {

				// Player 1 = Maximizer, so start low
				newBestHeuristic = Integer.MIN_VALUE;
			} else {

				// Player 2 = Minimizer, so start high
				newBestHeuristic = Integer.MAX_VALUE;
			}
			
			for(int i = 0; i < bestTurns.size(); i++) {

				int currentBestHeuristic = bestTurnsHeuristic.get(i);
				int currentTurn = bestTurns.get(i);
									
				// Set the turn
				field.doTurn(currentTurn, playerState);
				
				// Calculate the new heuristic for this state
				HeuristicResult fieldHeuristic = calcHeuristic(field);
				
				// Depending on our player number check which is the new best
				if(playerState == SlotState.PLAYER_1) {

					// Player 1 = Maximizer
					if(fieldHeuristic.heuristicGlobal > newBestHeuristic) {
						newBestHeuristic = fieldHeuristic.heuristicGlobal;
						newBestTurn = currentTurn;
					}
				} else {

					// Player 2 = Minimizer
					if(fieldHeuristic.heuristicGlobal < newBestHeuristic) {
						newBestHeuristic = fieldHeuristic.heuristicGlobal;
						newBestTurn = currentTurn;
					}
				}
				
				// Also remember both infos for the history
				reBestTurnsHeuristic.add(fieldHeuristic.heuristicGlobal);
				reBestTurns.add(currentTurn);
				
				// Undo the turn
				field.undoTurn();
			}
			
			// Set the new best turn as the turn to take
			if(newBestTurn != -1) {
				bestTurnCol = newBestTurn;
			}
			
			if (localLogger.isEnabledFor(Level.INFO)) {
				localLogger.log(Level.INFO, "CPUPlayer - Second evaluation stage complete");
				localLogger.log(Level.INFO, "Determined re-evaluated best turns (col / heuristic) : ");
			}
			
			// Display all currently determined turns
			for (int i = 0; i < reBestTurns.size(); i++) {
				
				// Mark the turn of choice with an arrow
				if(reBestTurns.get(i) == bestTurnCol) {
					if (localLogger.isEnabledFor(Level.INFO)) {
						localLogger.log(Level.INFO, "     " + reBestTurns.get(i) + " / " + reBestTurnsHeuristic.get(i) + " <=");
					}
				} else {
					if (localLogger.isEnabledFor(Level.INFO)) {
						localLogger.log(Level.INFO, "     " + reBestTurns.get(i) + " / " + reBestTurnsHeuristic.get(i));
					}
				}
			}
			
			if (localLogger.isEnabledFor(Level.INFO)) {
				localLogger.log(Level.INFO, "End of all re-evaluated best turns");
			}
			
			// Third stage, combine the old values with the new values
			int thirdBestHeuristic = 0;
			int thirdBestTurn = -1;
			
			// Init values depending on our player number
			if(playerState == SlotState.PLAYER_1) {

				// Player 1 = Maximizer, so start low
				thirdBestHeuristic = Integer.MIN_VALUE;
			} else {

				// Player 2 = Minimizer, so start high
				thirdBestHeuristic = Integer.MAX_VALUE;
			}
			
			for(int i = 0; i < reBestTurns.size(); i++) {
				
				int currentCombinedHeuristic = bestTurnsHeuristic.get(i) + reBestTurnsHeuristic.get(i);
				
				// Depending on our player number check which is the new best
				if(playerState == SlotState.PLAYER_1) {

					// Player 1 = Maximizer
					if(currentCombinedHeuristic > thirdBestHeuristic) {
						thirdBestHeuristic = currentCombinedHeuristic;
					}
				} else {

					// Player 2 = Minimizer
					if(currentCombinedHeuristic < thirdBestHeuristic) {
						thirdBestHeuristic = currentCombinedHeuristic;
					}
				}
			}
			
			// Check which turns are meeting the third best heuristic
			ArrayList<Integer> bestThirdTurns = new ArrayList<Integer>();
			
			for(int i = 0; i < reBestTurns.size(); i++) {
				int currentCombinedHeuristic = bestTurnsHeuristic.get(i) + reBestTurnsHeuristic.get(i);
				
				if(currentCombinedHeuristic == thirdBestHeuristic) {
					bestThirdTurns.add(reBestTurns.get(i));
				}
			}
			
			// Check if we have more than one equal turn
			int turnIndex = (int)(Math.random() * bestThirdTurns.size());
			thirdBestTurn = bestThirdTurns.get(turnIndex);
			
			if(thirdBestTurn != -1) {
				bestTurnCol = thirdBestTurn;
			}
						
			// Return the new heuristic
			return thirdBestHeuristic;
		}

		return bestHeuristic;
	}

	/**
	 * MAX Search Method (call first if Player 1)
	 * @param field the current playfield
	 * @param depth the current depth
	 * @return the best heuristic
	 */
	protected int max(Playfield field, int depth) {

		// Init the currently best heuristic for the maximizer
		int bestHeuristic = Integer.MIN_VALUE;

		// Calculate the current heuristic
		HeuristicResult heuristic = calcHeuristic(field);
		
		// If minimizer has won
		if(heuristic.heuristicPlayer2 <= -HVALUE_FULL_GROUP) {
			return heuristic.heuristicPlayer2;
		}
		
		// If max depth has been reached we reached a leaf in our tree
		if (depth == maxTreeDepth) {
			return heuristic.heuristicGlobal;
		}

		// Determine all currently legal turns
		int[] allLegalTurnCols = field.getAllLegalTurns();

		// Go through all legal turns
		for (int colIndex = 0; colIndex < allLegalTurnCols.length; colIndex++) {

			// Determine the currently legal turn by col and row
			int col = allLegalTurnCols[colIndex];
			int row = field.getMostBottomRow(col);

			// Set the turn
			field.setState(row, col, SlotState.PLAYER_1);

			// Let the other player have its turn
			int value = min(field, depth+1);
			
			// Undo the turn
			field.setState(row, col, SlotState.EMPTY);
			
			// Check if minimizer won
			if(value <= -HVALUE_FULL_GROUP) {
				return value;
			}

			// Check if the value is greater than the best heuristic
			if (value > bestHeuristic) {
				bestHeuristic = value;
			}
		}

		return bestHeuristic;
	}

	/**
	 * MIN Search Method (call first if Player 2)
	 * @param field the current play field
	 * @param depth the current depth
	 * @return the best heuristic
	 */
	protected int min(Playfield field, int depth) {

		// Init the best heuristic for the Minimizer
		int bestHeuristic = Integer.MAX_VALUE;

		// Calculate the current heuristic
		HeuristicResult heuristic = calcHeuristic(field);
		
		// If maximizer has won
		if(heuristic.heuristicPlayer1 >= HVALUE_FULL_GROUP) {
			return heuristic.heuristicPlayer1;
		}
		
		// Check if we reached the maximum depth
		if (depth == maxTreeDepth) {
			return heuristic.heuristicGlobal;
		}

		// Determine all legal turns
		int[] allLegalTurnCols = field.getAllLegalTurns();

		// Go through all legal turns
		for (int colIndex = 0; colIndex < allLegalTurnCols.length; colIndex++) {
			
			// Determine the currently legal turn by col and row
			int col = allLegalTurnCols[colIndex];
			int row = field.getMostBottomRow(col);
			
			field.setState(row, col, SlotState.PLAYER_2);

			// Let the other play have its turn
			int value = max(field, depth+1);
			
			// Undo the turn
			field.setState(row, col, SlotState.EMPTY);
			
			// Check if maximizer won
			if(value >= HVALUE_FULL_GROUP) {
				return value;
			}

			// Check if the value is smaller than the best heuristic
			if (value < bestHeuristic) {
				bestHeuristic = value;
			}
		}

		return bestHeuristic;
	}
	
	/**
	 * Calculate the current playfield heuristic (global, player1 and player2)
	 * @param field the actual playfield
	 * @return the heuristic result
	 */
	protected HeuristicResult calcHeuristic(Playfield field) {
		
		// The result
		HeuristicResult result = new HeuristicResult();
		
		for(int row = 0; row < field.getRows(); row++) {
			for(int col = 0; col < field.getCols(); col++) {
				
				// Test columns
				if(row <= (ApplicationConstants.PLAYFIELD_SIZE_ROWS - ApplicationConstants.PLAYFIELD_WIN_SIZE)) {
					
					result.heuristicPlayer1 += columnTest(field, col, row, NUM_PLAYER_1);
					result.heuristicPlayer2 += columnTest(field, col, row, NUM_PLAYER_2);
				}
				
				// Test rows
				if(col <= (ApplicationConstants.PLAYFIELD_SIZE_COLS - ApplicationConstants.PLAYFIELD_WIN_SIZE)) {
					
					result.heuristicPlayer1 += rowTest(field, col, row, NUM_PLAYER_1);
					result.heuristicPlayer2 += rowTest(field, col, row, NUM_PLAYER_2);
				}
				
				// Test falling diagonals (\)
				if(col <= (ApplicationConstants.PLAYFIELD_SIZE_COLS - ApplicationConstants.PLAYFIELD_WIN_SIZE) && row > (ApplicationConstants.PLAYFIELD_SIZE_ROWS - ApplicationConstants.PLAYFIELD_WIN_SIZE)) {
					
					result.heuristicPlayer1 += diagFallingTest(field, col, row, NUM_PLAYER_1);
					result.heuristicPlayer2 += diagFallingTest(field, col, row, NUM_PLAYER_2);
				}
				
				// Test raising diagonals (/)
				if(col <= (ApplicationConstants.PLAYFIELD_SIZE_COLS - ApplicationConstants.PLAYFIELD_WIN_SIZE) && row <= (ApplicationConstants.PLAYFIELD_SIZE_ROWS - ApplicationConstants.PLAYFIELD_WIN_SIZE)) {
					
					result.heuristicPlayer1 += diagRaisingTest(field, col, row, NUM_PLAYER_1);
					result.heuristicPlayer2 += diagRaisingTest(field, col, row, NUM_PLAYER_2);
				}
			}
		}
		
		// Apply the correct factors (Player1 - Maximizer, Player2 - Minimizer)
		result.heuristicPlayer2 = -1*result.heuristicPlayer2;
		result.heuristicGlobal = result.heuristicPlayer1 + result.heuristicPlayer2;
		
		return result;
	}

	/**
	 * Test the column from bottom to top
	 * @param field the playfield
	 * @param col the col to test from
	 * @param row the row to test from
	 * @param player the numerical player representation (Player1 = 1, Player2 = -1)
	 * @return the result value
	 */
	protected int columnTest(Playfield field, int col, int row, int player) {
		
		// Init data field
		int group = 0;
		
		for (int c = 0; c < ApplicationConstants.PLAYFIELD_WIN_SIZE; c++) {
			
			// Check if the field belongs to the enemy player or us
			if (field.getStateNum(row + c, col) == -player) {
				return (0);
				
			} else if (field.getStateNum(row + c, col) == player) {
				group++;
			}
		}
		
		// Determine the heuristic based on the group size
		if(group == (ApplicationConstants.PLAYFIELD_WIN_SIZE-2)) {
			return this.HVALUE_TWO_MISSING;
			
		} else if(group == (ApplicationConstants.PLAYFIELD_WIN_SIZE-1)) {
			return this.HVALUE_ONE_MISSING;
			
		} else if(group == ApplicationConstants.PLAYFIELD_WIN_SIZE) {
			return this.HVALUE_FULL_GROUP;
			
		} else {
			// Small group, no points
			return 0;
		}
	}

	/**
	 * Test the row from left to the right
	 * @param field the playfield
	 * @param col the col to test from
	 * @param row the row to test from
	 * @param player the numerical player representation (Player1 = 1, Player2 = -1)
	 * @return the result value
	 */
	protected int rowTest(Playfield field, int col, int row, int player) {

		// Init data fields
		int group = 0;
		int obstacles = 0;

		for (int c = 0; c < ApplicationConstants.PLAYFIELD_WIN_SIZE; c++) {
			
			// Check if the field belongs to the enemy player, us or is empty
			if (field.getStateNum(row, col + c) == -player) {
				return (0);
				
			} else if (field.getStateNum(row, col + c) == player) {
				group++;
				
			} else if (field.getStateNum(row, col + c) == 0) {
				
				// Add the number of stones missing under the current one
				obstacles += getNumberOfMissingStonesUnderPos(field, col + c, row) - row;
			}
		}
		
		// Give more weight to the obstacles
		obstacles *= 2;

		// Determine the heuristic based on the group size
		if(group == (ApplicationConstants.PLAYFIELD_WIN_SIZE-2)) {
			return this.HVALUE_TWO_MISSING;
			
		} else if(group == (ApplicationConstants.PLAYFIELD_WIN_SIZE-1)) {
			return this.HVALUE_ONE_MISSING;
			
		} else if(group == ApplicationConstants.PLAYFIELD_WIN_SIZE) {
			return this.HVALUE_FULL_GROUP;
			
		} else {
			// Small group, no points
			return 0;
		}
	}

	/**
	 * Test the raising diagonal (/) from left to the right
	 * @param field the playfield
	 * @param col the col to test from
	 * @param row the row to test from
	 * @param player the numerical player representation (Player1 = 1, Player2 = -1)
	 * @return the result value
	 */
	protected int diagRaisingTest(Playfield field, int col, int row, int player) {

		// Init data fields
		int group = 0;
		int obstacles = 0;

		// Check if there are enough connected items
		for (int c = 0; c < ApplicationConstants.PLAYFIELD_WIN_SIZE; c++) {
			
			// Check if the field belongs to the enemy player, us or is empty
			if (field.getStateNum(row + c, col + c) == -player) {
				return (0);
				
			} else if (field.getStateNum(row + c, col + c) == player) {
				group++;
				
			} else if (field.getStateNum(row + c, col + c) == 0) {
				
				// Add the number of stones missing under the current one
				obstacles += getNumberOfMissingStonesUnderPos(field, col + c, row + c) - (row+c);
			}
		}
		
		// Give more weight to the obstacles
		obstacles *= 2;

		// Determine the heuristic based on the group size
		if(group == (ApplicationConstants.PLAYFIELD_WIN_SIZE-2)) {
			return this.HVALUE_TWO_MISSING;
			
		} else if(group == (ApplicationConstants.PLAYFIELD_WIN_SIZE-1)) {
			return this.HVALUE_ONE_MISSING;
			
		} else if(group == ApplicationConstants.PLAYFIELD_WIN_SIZE) {
			return this.HVALUE_FULL_GROUP;
			
		} else {
			// Small group, no points
			return 0;
		}
	}

	/**
	 * Test the falling diagonal (\) from left to the right
	 * @param field the playfield
	 * @param col the col to test from
	 * @param row the row to test from
	 * @param player the numerical player representation (Player1 = 1, Player2 = -1)
	 * @return the result value
	 */
	protected int diagFallingTest(Playfield field, int col, int row, int player) {

		// Init the data fields
		int group = 0;
		int obstacles = 0;

		// Check if there are four connected items
		for (int c = 0; c < ApplicationConstants.PLAYFIELD_WIN_SIZE; c++) {
			
			// Check if the field belongs to the enemy player, us or is empty
			if (field.getStateNum(row - c, col + c) == -player) {
				return (0);
				
			} else if (field.getStateNum(row - c, col + c) == player) {
				group++;
			
			} else if (field.getStateNum(row - c, col + c) == 0) {
				
				// Add the number of stones missing under the current one
				obstacles += getNumberOfMissingStonesUnderPos(field, col + c, row - c) - (row-c);
			}
		}
		
		// Give more weight to the obstacles
		obstacles *= 2;

		// Determine the heuristic based on the group size
		if(group == (ApplicationConstants.PLAYFIELD_WIN_SIZE-2)) {
			return this.HVALUE_TWO_MISSING;
			
		} else if(group == (ApplicationConstants.PLAYFIELD_WIN_SIZE-1)) {
			return this.HVALUE_ONE_MISSING;
			
		} else if(group == ApplicationConstants.PLAYFIELD_WIN_SIZE) {
			return this.HVALUE_FULL_GROUP;
			
		} else {
			// Small group, no points
			return 0;
		}
	}
	
	/**
	 * Retrieve the number of missing stones under the specified position
	 * @param field the playfield
	 * @param col the col
	 * @param row the row
	 * @return the number of missing stones
	 */
	protected int getNumberOfMissingStonesUnderPos(Playfield field, int col, int row) {
				
		int mostBottomRow = field.getMostBottomRow(col);

		return (row - mostBottomRow);
	}

	/**
	 * Class Description:
	 * 		Internal class representing a heuristic state
	 */
	class HeuristicResult {

		// The global heuristic consisting of both the player1 and player2 heuristics
		public int heuristicGlobal = 0;

		// The player heuristics
		public int heuristicPlayer1 = 0;
		public int heuristicPlayer2 = 0;
		
		/**
		 * Constructor
		 */
		public HeuristicResult() {}

		/**
		 * Constructor
		 * @param heuristicGlobal
		 * @param heuristicPlayer1
		 * @param heuristicPlayer2
		 */
		public HeuristicResult(int heuristicGlobal, int heuristicPlayer1, int heuristicPlayer2) {
			this.heuristicGlobal = heuristicGlobal;
			this.heuristicPlayer1 = heuristicPlayer1;
			this.heuristicPlayer2 = heuristicPlayer2;
		}
	}

	/**
	 * Retrieve the set max tree depth
	 * @return
	 */
	public int getMaxTreeDepth() {
        	return maxTreeDepth;
        }

	/**
	 * Set a new max tree depth
	 * @param maxTreeDepth
	 */
	public void setMaxTreeDepth(int maxTreeDepth) {
        	this.maxTreeDepth = maxTreeDepth;
        }
}
